;*************************************************************
;
; Copyright (c) 2000 Todd Elliott
; IR Decoder/Controller

; This program is free software; you can redistribute it and/or
; modify it under the terms of the GNU General Public License
; as published by the Free Software Foundation; either version 2
; of the License, or (at your option) any later version.

; This program is distributed in the hope that it will be useful,
; but WITHOUT ANY WARRANTY; without even the implied warranty of
; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
; GNU General Public License for more details.

; You should have received a copy of the GNU General Public License
; along with this program; if not, write to the Free Software
; Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

; Contact: tvelliott@yahoo.com 

;
; $Log: mp3ircontroller.asm,v $
; Revision 1.9  2000/09/30 04:31:46  telliott
; got 32 buttons working!
;
; Revision 1.8  2000/09/26 13:04:28  telliott
; updated to new serial commands
;
; Revision 1.7  2000/09/26 04:22:36  telliott
; *** empty log message ***
;
; Revision 1.6  2000/09/26 04:06:25  telliott
; updated Documentation some more.  Added sample output from decoder.
;
; Revision 1.5  2000/09/26 03:37:07  telliott
; added some documentation for the project in the source.
;
; Revision 1.4  2000/09/26 03:05:17  telliott
; cleaned up code a bit and added some more comments.
;
; Revision 1.3  2000/09/26 02:33:57  telliott
; made the ir code visual representation horizontal instead of vertical.
;
; Revision 1.2  2000/09/25 09:54:17  telliott
; Got it working with 5 different remotes flawless.
;
; Revision 1.1  2000/09/25 06:16:09  telliott
; IR controller for www.pjrc.com mp3 player.
;
;
; $Revision: 1.9 $ 
;
include "p16c84.inc"		;Microchips register equate file


       __CONFIG  _CP_OFF&_WDT_OFF&_XT_OSC&_PWRTE_ON;


PIC84	equ	00h
B192_DEL	equ	0ch		;19200 Baud Delay constants
B192_HDEL	equ	02h

tpin	equ	5 	;port B		serial out
rpin 	equ	4 	;port B		serial in

ir_in   equ	2	;port A		infrared input
led_out equ	3	;port B		indicator led (may be disabled)
			;
			;I found that the LED may cause problems with the
			;AGC circuit in the IR decoder module.  This may
			;just be a problem with supply current.  I had 
			;problems when using the PC serial port to power
			;the board.  This may not be the case when using
			;the mp3 player to power it.

delcnt1		equ	00dh	;various storage for serial routines
delcnt2		equ	00eh
xcount		equ	00fh
xmtreg		equ	010h

keycode		equ	011h
ircode_1	equ	012h
ircode_2	equ	013h
button_num_1	equ	014h
button_num_2	equ	015h

textpt		equ	016h	;for serial output routine
mscnt		equ	017h

temp 		equ	018h	;more temp registers
temp2		equ	019h

		;free

ircount		equ	01Bh
temp3		equ	01Ch
flags		equ	01Dh		;various flags are stuffed into
					;this ram location
button_flag	equ	0
learn_flag	equ	1
match_flag	equ	2
first_key	equ	3
long_delay	equ	4



start_byte	equ	020h		;start of ram for temporary storage
					;of IR code before sending to the
					;EEProm


	org	PIC84	
	goto	init


	org	0004h		;interrupt vector
	goto 	IntHandler
;**************************************************************
;		INITIALIZATION
;**************************************************************

	org	0005h

serout:
	bcf	INTCON,GIE		;disable interrupts for IR
	movwf	textpt
seroutlp
        movlw LOW TEXTDATA
        addwf textpt,w
        movlw HIGH TEXTDATA
        btfsc STATUS,C
        addlw 1
        movwf PCLATH
          
        movf   textpt,w
        call   TEXTDATA

	movwf	xmtreg
	subwf	xmtreg,w	;test for zero
	movf	xmtreg,w
	btfsc	STATUS,Z
	goto	seroutend
	call	trmit
	incf	textpt
	goto	seroutlp
seroutend:
	bcf	INTCON,RBIF		;clear portB interrupt flag
	bsf	INTCON,GIE		;disable interrupts for IR
	return

TEXTDATA:		;text message table
	addwf	PCL
text_offset:

press_mess:
	retlw	'P'
	retlw	'R'
	retlw	'E'
	retlw	'S'
	retlw	'S'
	retlw	' '
	retlw	00h

done_mess:
	retlw	'D'
	retlw	'O'
	retlw	'N'
	retlw	'E'
	retlw	' '
	retlw	00h

init_mess:
	retlw	'I'
	retlw	'R'
	retlw	' '
	retlw	'D'
	retlw	'E'
	retlw	'C'
	retlw	'O'
	retlw	'D'
	retlw	'E'
	retlw	00h

play_mess:
	retlw	'P'
	retlw	'L'
	retlw	'A'
	retlw	'Y'
	retlw	00h
next_mess:
	retlw	'N'
	retlw	'E'
	retlw	'X'
	retlw	'T'
	retlw	00h
previous_mess:
	retlw	'P'
	retlw	'R'
	retlw	'E'
	retlw	'V'
	retlw	'I'
	retlw	'O'
	retlw	'U'
	retlw	'S'
	retlw	00h
random_mess:
	retlw	'R'
	retlw	'A'
	retlw	'N'
	retlw	'D'
	retlw	'O'
	retlw	'M'
	retlw	00h



init:
	bcf	INTCON,GIE

	bcf		INTCON,INTE	;disable pinb0 int
	bsf		INTCON,RBIE	;disable port b ints
	bcf		INTCON,T0IE	;disable rtcc int
	bcf		INTCON,EEIE	;disable eeprom int

	bsf		STATUS,RP0	;select page 1
	movlw	010h		;input on pin RB4 
	movwf	TRISB 		;updata data direction
	movlw	004h	        ;input on pin RA2
	movwf	TRISA 		;updata data direction
	movlw	007h		;rtcc internal with 1/256 prescaler
	movwf	OPTION_REG 	;option register
	bcf	EECON1,WREN	;disable write to eeprom	
	bcf	STATUS,RP0	;select page 0


	bcf		PORTB,tpin ;turn off start bit

	bsf	PORTB,led_out

	bsf	flags,long_delay
	call	crlf
	movlw	(init_mess-TEXTDATA)-1		;INIT message
	call	serout
	call	crlf

	bcf	INTCON,RBIF
	bsf	INTCON,GIE	;enable interrupts for RBIE

	bcf	flags,button_flag
	bcf	flags,learn_flag
	bsf	flags,first_key
	goto	main



;**************************************************************
;	MAIN LOOP
;**************************************************************
ds45us:			;45 us Delay for filtering background IR
	movlw	5
	movwf	delcnt1
ds45lp:
	decfsz	delcnt1
	goto	ds45lp
	return	

MSdel:					;delay W milliseconds
	movwf	mscnt
msd1:
	call	Delms
	decfsz	mscnt
	goto	msd1
	return

Delms:					;delay for 1 ms
	movlw	01h
	movwf	delcnt2
del2:
	clrf	delcnt1
del1:
	decfsz	delcnt1
	goto	del1
	decfsz	delcnt2
	goto	del2
	movlw	100	
	movwf	delcnt1
del3:
	decfsz	delcnt1
	goto	del3
	return

bad_key:
	bcf	flags,button_flag
	bcf	flags,learn_flag
main:
	btfsc	flags,button_flag	;is there a keyboard button press
	goto	get_b_code	

	btfsc	PORTA,ir_in
	goto	get_code
	goto	main


get_b_code:
	bsf	flags,learn_flag		;learn a new key on remote
	bcf	flags,button_flag		;clear button flag
	bsf	flags,first_key

	movlw	']'
	subwf	button_num_1,w
	btfss	STATUS,Z
	goto	bad_key	
	bsf	flags,long_delay
	movlw	(press_mess-TEXTDATA)-1		;PRESS message
	call	serout
	movlw	100
	call	MSdel
	bcf	flags,button_flag

wait_key_code_1:
	btfss	flags,button_flag
	goto	wait_key_code_1		;really should timeout here

	bcf	flags,button_flag		;clear button flag

	bcf	flags,first_key
wait_key_code_2:
	btfss	flags,button_flag
	goto	wait_key_code_2		;really should timeout here

	bcf	INTCON,GIE		;disable interrupts for IR
	bcf	flags,button_flag		;clear button flag
	bsf	flags,first_key

	movlw	'L'
	call	trmit		;learn a command

get_code:
	movlw	10
	movwf	ircount
	call	clear_fsr
	;bsf	PORTB,led_out
main_1
	call	Delms	
	btfss	PORTA,ir_in
	goto	main
	call	ds45us			;filter background IR
					;and make sure we are synced with
					;beginning of signal
	btfss	PORTA,ir_in
	goto	main
	btfsc	flags,button_flag
	goto	get_b_code
	decfsz	ircount
	goto	main_1



	movlw	2
	movwf	ircount		;do twice to get last part of IR code
fsrloop				;we dont have enough ram for whole thing
	movlw	start_byte
	movwf	FSR		;setup indirect address for code

wait_ir1
	btfsc	flags,button_flag
	goto	get_b_code
	btfsc	PORTA,ir_in
	goto	wait_ir1
	call	ds45us			;look for background ir 
	btfsc	PORTA,ir_in
	goto	wait_ir1

	;bcf	PORTB,led_out

wirloop
	movlw	1
	movwf	TMR0
wir1
	btfsc	flags,button_flag
	goto	get_b_code
	movf	TMR0,w
	btfsc	STATUS,Z
	goto	get_code		;were done ... timeout
	btfss	PORTA,ir_in
	goto	wir1
	call	ds45us			;look for background ir 
	btfss	PORTA,ir_in
	goto	wir1

	;bsf	PORTB,led_out

	movf	TMR0,w	
	sublw	5
	movwf	temp
	movlw	05h
	btfss	temp,7			;is the bit-time less than
					;5x256*1us?  If so, we will
					;represent it with a 5
	goto	store_bit1
	movlw	0ah			;otherwise we will represent it
					;with a ten
store_bit1
	movwf	INDF  ;indirect FSR	;low nibble for low time


wirloop2
	movlw	1
	movwf	TMR0
wir2
	btfsc	flags,button_flag
	goto	get_b_code
	movf	TMR0,w
	btfsc	STATUS,Z
	goto	get_code		;where done timeout
	btfsc	PORTA,ir_in
	goto	wir2

	;bcf	PORTB,led_out

	movf	TMR0,w			;same as above but for the high time
	sublw	5
	movwf	temp
	movlw	05h
	btfss	temp,7
	goto	store_bit2
	movlw	0ah
store_bit2
	movwf	temp

	swapf	temp,w		
	iorwf	INDF			;high nibble for high time

	incf	FSR			;next address

	btfss	FSR,4		
	goto	wirloop	
	btfss	FSR,5			;check for last location 0x2f
	goto	wirloop	

	decfsz	ircount   ;do twice to get the last part of the code
			  ;we don't have enough ram for the whole thing
	goto	fsrloop

	;bsf	PORTB,led_out

	movlw	start_byte
	movwf	FSR


	btfss	flags,learn_flag
	goto	end_outloop

	call	crlf
	movlw	0ah
	movwf	temp
outloop			;output visual display of bit times
			;using a '|' character

	movf	INDF,w
	andlw	08h
	movwf	temp3

	movf	temp3,w
	movlw	'|'
	btfss	STATUS,Z
	goto	one_bit1	
	movlw	05h
	subwf	temp,w
	movwf	temp2
	movlw	' '
	btfss	temp2,7
	goto	one_bit1
	movlw	'|'
one_bit1
	call	trmit
	movlw	' '
	call	trmit

	swapf	INDF,w
	andlw	08h		;is it a '5' bit or a '10' bit?
	movwf	temp3

	movf	temp3,w
	movlw	'|'
	btfss	STATUS,Z	;if we are within the 1st 5 lines of output
	goto	one_bit2	;output a '|' for '10' bits and ' ' for '5'
	movlw	05h
	subwf	temp,w
	movwf	temp2
	movlw	' '
	btfss	temp2,7	;are we past the first 5 lines of output?
	goto	one_bit2	;if so always output a '|' char
	movlw	'|'
one_bit2
	call	trmit
	movlw	' '
	call	trmit

	incf	FSR

	btfss	FSR,4		;check for last reg 0x2f
	goto	outloop	
	btfss	FSR,5		
	goto	outloop	
	movlw	start_byte
	movwf	FSR
	call	crlf
	decfsz	temp
	goto	outloop
end_outloop

	btfss	flags,learn_flag
	goto	find_match

;key codes are in button_num_1 and button_num2

	movlw	'0'
	subwf	button_num_1,w
	movwf	keycode
	swapf	keycode

	movlw	'0'
	subwf	button_num_2,w
	iorwf	keycode		;we now should have 2 digit bcd number in 
				;keycode
	movf	keycode,w
	call	bcd2bin		;convert to binary
	movwf	keycode		;save binary command code


	bcf	STATUS,C    	;make sure not to roll in one bits
	rlf	keycode,w	;multiply by 2 for offset into EEPROM	

	movwf	EEADR		;EEPROM address to write to


	call	convert_code	;can use this for match code also

	movf	temp,w
	call	save_code	;EEADR should already be set 
	movf	temp2,w
	call	save_code	;EEADR is incremented in save_code

	goto	end_learn

;************************************************************
;       SERIAL ROUTINES
;************************************************************

crlf:
	movlw	0dh		;carriage return / line feed
	call	trmit
	movlw	0ah
	call	trmit
	return

trmit:				;transmit byte in xmtreg to serial out
	bcf	INTCON,GIE
	movwf	xmtreg
	bsf	PORTB,tpin	;start bit
	movlw	8
	movwf	xcount
	call	Delay
tr_1
	btfsc	xmtreg,0
	goto	tr_2
	bsf	PORTB,tpin
	goto	tr_3
tr_2
	bcf	PORTB,tpin
tr_3
	rrf	xmtreg,1
	call	Delay
	decfsz	xcount
	goto	tr_1
	nop
	nop
	bcf	PORTB,tpin
	call	Delay
	call	Delay
	btfss	flags,long_delay
	goto	end_trmit
	call	Delay
	call	Delay
	call	Delay
	call	Delay
	call	Delay
	call	Delay
	call	Delay
	call	Delay
	call	Delay
	call	Delay
	call	Delay

end_trmit
	bcf	flags,button_flag		;clear button flag
	return


Recv:				;rec byte from serial into xmtreg
	call	HalfDelay
	btfss	PORTB,rpin	;are we too late?	
	return

	movlw	8
	movwf	xcount
	nop
	nop
rec_1
	call	Delay
	btfsc	PORTB,rpin	
	goto	rec_2
	bsf	xmtreg,7
	goto	rec_3
rec_2
	bcf	xmtreg,7
rec_3
	decfsz	xcount,w
	rrf	xmtreg,1
	decfsz	xcount
	goto	rec_1
	call	Delay
	btfsc	PORTB,rpin
	clrf	xmtreg
	return

Delay:					;delay constant for xxxx baud
	movlw	B192_DEL
	movwf	delcnt1
	nop
	nop
dellp:
	decfsz	delcnt1
	goto	dellp
	return


HalfDelay:				;delay constant for xxxx baud
	movlw	B192_HDEL	
	movwf	delcnt2
	nop
	nop
hlfdellp
	decfsz	delcnt2
	goto	hlfdellp
	return

;*****************************************************************
; This is where we compress the ir code to 2 bytes  temp and temp2
;*****************************************************************

convert_code
	movlw	start_byte	
	movwf	FSR	

	movlw	08h
	movwf	delcnt1

encode_1
	movf	INDF,w
	sublw	055h		;it is either 55h or 5Ah  zero/one...
	btfss	STATUS,Z
	goto	ircd_1

	bcf	STATUS,C	;zero bit
	goto	ircd_rot
ircd_1
	bsf	STATUS,C	;one  bit
ircd_rot
	rrf	temp
	incf	FSR
	decfsz	delcnt1
	goto	encode_1

	movlw	08h
	movwf	delcnt1

encode_2
	movf	INDF,w
	sublw	055h		;it is either 55h or 5Ah  zero/one...
	btfss	STATUS,Z
	goto	ircd2_1

	bcf	STATUS,C	;zero bit
	goto	ircd2_rot
ircd2_1
	bsf	STATUS,C	;one  bit
ircd2_rot
	rrf	temp2
	incf	FSR
	decfsz	delcnt1
	goto	encode_2
	return

	
save_code
	movwf	EEDATA
	bsf		STATUS,RP0	;select page 1
	bcf	EECON1,EEIF
	bsf	EECON1,WREN	
	movlw	55h
	movwf	EECON2
	movlw	0AAh
	movwf	EECON2
	bsf	EECON1,WR		;start write
wait_wr
	btfss	EECON1,EEIF
	goto	wait_wr
	bcf	EECON1,EEIF
	bcf		STATUS,RP0	;select page 0

	incf	EEADR

	bsf		STATUS,RP0	;select page 1
	bcf	EECON1,WREN		;disable write to eeprom	
	bcf		STATUS,RP0	;select page 0
	return


end_learn
	call	crlf
	bcf	flags,learn_flag

	bsf	flags,long_delay
	movlw	(done_mess-TEXTDATA)-1	;DONE message
	call	serout

	call	crlf
	goto	end_ir

find_match
	bcf	flags,long_delay
	call	convert_code	;get code in temp and temp2

				;current button to check
	clrf	temp3
find_m1
	movf	temp3,w
	movwf	EEADR		;sixteen byte boundries

	movlw	temp		;start of indirect
	movwf	FSR
	movlw	02h
	movwf	delcnt2
find_m2
	bsf		STATUS,RP0	;select page 1
	bsf	EECON1,RD		;read byte from eeprom
	bcf		STATUS,RP0	;select page 0
	movf	INDF,w
	subwf	EEDATA,w
	btfss	STATUS,Z
	goto	no_match
	incf	EEADR
	incf	FSR
	decfsz	delcnt2
	goto	find_m2

				;found match in temp!
	movlw	0	
	subwf	temp3,w
	btfss	STATUS,Z
	goto	chk_one
	movlw	'0'	
	call	trmit
	goto	end_ir	
chk_one
	movlw	2	
	subwf	temp3,w
	btfss	STATUS,Z
	goto	chk_two
	movlw	'1'	
	call	trmit
	goto	end_ir	
chk_two
	movlw	4	
	subwf	temp3,w
	btfss	STATUS,Z
	goto	chk_three
	movlw	'2'	
	call	trmit
	goto	end_ir	
chk_three
	movlw	6	
	subwf	temp3,w
	btfss	STATUS,Z
	goto	chk_four
	movlw	'3'	
	call	trmit
	goto	end_ir	
chk_four
	movlw	8	
	subwf	temp3,w
	btfss	STATUS,Z
	goto	chk_five
	movlw	'4'	
	call	trmit
	goto	end_ir	
chk_five
	movlw	0ah	
	subwf	temp3,w
	btfss	STATUS,Z
	goto	chk_six
	movlw	'5'	
	call	trmit
	goto	end_ir	
chk_six
	movlw	0ch	
	subwf	temp3,w
	btfss	STATUS,Z
	goto	chk_seven
	movlw	'6'	
	call	trmit
	goto	end_ir	
chk_seven
	movlw	0eh	
	subwf	temp3,w
	btfss	STATUS,Z
	goto	chk_eight
	movlw	'7'	
	call	trmit
	goto	end_ir	
chk_eight
	movlw	010h	
	subwf	temp3,w
	btfss	STATUS,Z
	goto	chk_nine
	movlw	'8'	
	call	trmit
	goto	end_ir	
chk_nine
	movlw	012h	
	subwf	temp3,w
	btfss	STATUS,Z
	goto	chk_play
	movlw	'9'	
	call	trmit
	goto	end_ir	

chk_play
	movlw	014h	
	subwf	temp3,w
	btfss	STATUS,Z
	goto	chk_next	
	movlw	(play_mess-TEXTDATA)-1	
	goto	send_command	;Play/Pause

chk_next
	movlw	016h	
	subwf	temp3,w
	btfss	STATUS,Z
	goto	chk_last	
	movlw	(next_mess-TEXTDATA)-1	
	goto	send_command	;Next File

chk_last
	movlw	018h	
	subwf	temp3,w
	btfss	STATUS,Z
	goto	chk_random
	movlw	(previous_mess-TEXTDATA)-1	
	goto	send_command	;Last File
chk_random
	movlw	01ah	
	subwf	temp3,w
	btfss	STATUS,Z
	goto	end_ir
	movlw	(random_mess-TEXTDATA)-1	
	goto	send_command	;Randomize

;**************************************************

	;could add more buttons here

;**************************************************

send_command
	call	serout		;transmit command to mp3 player 
	call	crlf
	goto	end_ir

no_match
	incf	temp3
	incf	temp3
	movlw	020h
	subwf	temp3,w
	btfss	STATUS,Z
	goto	find_m1
				;if we didn't find it in 32 tries
				;we didn't find a match...

	

end_ir
	bcf	INTCON,RBIF	;clear portB interrupt flag
	bsf	INTCON,GIE
	movlw	150
	call	MSdel
	bcf	flags,button_flag		;clear button flag
	goto	main	

clear_fsr			;clear 16 memory locations for IR code
	movlw	start_byte
	movwf	FSR
clear_loop
	clrf	INDF
	incf	FSR
	btfss	FSR,4		
	goto	clear_loop	
	btfss	FSR,5		;check for last reg 0x2f
	goto	clear_loop	
	return




;**************************************************************
; 	INTERRUPT HANDLER
;**************************************************************

IntHandler:
	btfsc	INTCON,RBIF
	goto	handportb	;serial input int

handportb:
	call	Recv		;get a byte from serial port
	movf	xmtreg,w	
	btfss	flags,first_key
	goto	i_b1
	movwf	button_num_1
	goto	int_done
i_b1:	
	movwf	button_num_2
	
int_done:
	bcf	INTCON,RBIF	;clear portB interrupt flag
	bsf	flags,button_flag

endint:
	retfie

;********************************************************************
bcd2bin
      movwf	temp 
      clrf	temp2
      andlw	0xF0
      movwf     temp3
      swapf	temp3
      movf	temp3,w
      btfsc	STATUS,Z
      goto	skipbcdbin1
bcdbin1
	movf	temp2,w
	addlw   0ah
	movwf	temp2
	decfsz  temp3 
	goto    bcdbin1

skipbcdbin1
	movf	temp,w
	andlw	0x0F
	movwf	temp3
	movf	temp3,w
	btfsc	STATUS,Z
	goto    end_bcd2bin
bcdbin2
	incf	temp2
	decfsz	temp3
	goto	bcdbin2
      
end_bcd2bin
        movf temp2,w
	return

	end
